/*
 * Generated by MyEclipse Struts
 * Template path: templates/java/JavaClass.vtl
 */
package gov.va.med.vistalink.management.action;

import gov.va.med.vistalink.console.IServerHelper;
import gov.va.med.vistalink.console.ServerHelperFactory;
import gov.va.med.vistalink.jmx.IJmxHelper;
import gov.va.med.vistalink.jmx.JmxHelperException;
import gov.va.med.vistalink.jmx.JmxHelperFactory;
import gov.va.med.vistalink.management.ServerTO;

import java.net.URLEncoder;
import java.util.Iterator;
import java.util.Set;
import java.util.TreeSet;

import javax.management.MalformedObjectNameException;
import javax.management.ObjectName;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

//ArM - 06/20/2018 - #TRM
import org.apache.logging.log4j.Logger;
import org.apache.logging.log4j.LogManager;
import org.apache.struts.Globals;
import org.apache.struts.action.ActionErrors;
import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;
import org.apache.struts.action.ActionMessage;
import org.apache.struts.action.ActionMessages;

/**
 * Struts action to populate console main page, including listing servers.
 * 
 */
public class ConsoleMainAction extends VistaLinkActionBase {

	private static final Logger logger = LogManager.getLogger(ConsoleMainAction.class);

	/**
	 * Override default roles, add MONITOR to allow viewing top-level server choice page
	 */
	private static final byte NORMAL_ROLES_PLUS_MONITOR = VistaLinkActionBase.DEPLOYER_OR_ADMIN_OR_OPERATOR | ROLE_MONITOR ;

	//text vars
	private static final String TEXT_PROB_DOMAIN_SERVERS = "Could not get sorted domain servers.";
	
	/*
	 * main console action/page
	 */

	/**
	 * Method execute
	 * @param mapping
	 * @param form
	 * @param request
	 * @param response
	 * @return ActionForward
	 */
	public ActionForward execute(ActionMapping mapping, ActionForm form, HttpServletRequest request,
			HttpServletResponse response) {

		addPageRenderedTime(request);
		addSessionTimeoutWarning(request);

		// security check
		try {
			checkIsUserInRole(request, NORMAL_ROLES_PLUS_MONITOR);
		} catch (SecurityException e) {
			logger.warn("Access attempt without sufficient privileges" + e.getMessage());
			ActionErrors errors = getGlobalActionErrors(request);
			errors.add(ActionMessages.GLOBAL_MESSAGE, new ActionMessage("configurator.nopriv", new Object[] { e.getMessage() }));
			request.setAttribute(Globals.ERROR_KEY, errors);
			return mapping.findForward("nopriv");
		}
		
		// get list of server names to display on main page
		try {
			Set serverHelpers = getSortedDomainServers();
			Iterator serverIter = serverHelpers.iterator();
			Set serverSet = new TreeSet();
			while (serverIter.hasNext()) {
				IServerHelper serverHelper = (IServerHelper) serverIter.next();
				ServerTO serverTO = new ServerTO();
				serverTO.setServerConstructorAttributes(URLEncoder.encode(serverHelper.getServerHelperConstructorProperties(), "UTF-8"));
				serverTO.setServerName(serverHelper.getDisplayNameFullyLocationQualified());
				serverSet.add(serverTO);
			}
			request.setAttribute("serverset", serverSet);
		} catch (Exception e) {
			logger.error("Could not list servers: ", e);
			ActionErrors actionErrors = getGlobalActionErrors(request);
			actionErrors.add(ActionMessages.GLOBAL_MESSAGE, new ActionMessage("no.serverlist"));
			request.setAttribute(Globals.ERROR_KEY, actionErrors);
		}
		return mapping.findForward("success");
	}
	
	/*
	 * private method to return a sorted set of the servers in the (weblogic) domain
	 */
	private Set getSortedDomainServers() throws JmxHelperException {
		logger.debug("in getSortedDomainServers");
		Set serverSet = new TreeSet();
		IJmxHelper jmxHelper = JmxHelperFactory.getJmxHelper();
		try {
			ObjectName serverQueryObjectName = new ObjectName(jmxHelper.getJmxDomainForPlatform() + ":Type=Server,*");
			Set serverObjectNameSet = jmxHelper.getLocalMBeanServer().queryNames(serverQueryObjectName, null);
			Iterator serverIter = serverObjectNameSet.iterator();
			while (serverIter.hasNext()) {
				ObjectName serverObjectName = (ObjectName) serverIter.next();
				IServerHelper serverHelper = ServerHelperFactory.getServerHelper(serverObjectName
						.getCanonicalKeyPropertyListString());
				serverSet.add(serverHelper);
			}
		} catch (MalformedObjectNameException e) {
			logger.error(TEXT_PROB_DOMAIN_SERVERS, e);
			throw new JmxHelperException(TEXT_PROB_DOMAIN_SERVERS, e);
		} catch (NullPointerException e) {
			logger.error(TEXT_PROB_DOMAIN_SERVERS, e);
			throw new JmxHelperException(TEXT_PROB_DOMAIN_SERVERS, e);
		}
		return serverSet;
	}

}