package gov.va.med.vistalink.fortify;

//ArM - 06/20/2018 - #TRM
import org.apache.logging.log4j.Logger;

import java.net.UnknownHostException;
import java.net.InetAddress;
import org.apache.logging.log4j.LogManager;

public final class Factory {
	/**
	 * The logger used by this class
	 */
	private static final Logger logger = LogManager.getLogger(Factory.class);	
	
	/*
	 * Checks for formatting and lack of leading 0.
	 * Trying to avoid using the InetAddress.getName() derivatives to not trigger the Fortify.
	 * 
	 */
	protected final static boolean isValidIP(final String ip) {
	    String PATTERN = "^((0|1\\d?\\d?|2[0-4]?\\d?|25[0-5]?|[3-9]\\d?)\\.){3}(0|1\\d?\\d?|2[0-4]?\\d?|25[0-5]?|[3-9]\\d?)$";

	    return ip.matches(PATTERN);
	}
	/* 
	 * ArM
	 * 05-17-2018
	 * 
	 * #FortifyMadness
	 * 
	 */
	
	/* Do a reverse DNS lookup to find the host name associated with an IP address. 
	 * Gets results more often than
	 * {@link java.net.InetAddress#getCanonicalHostName()}, 
	 * but also tries the Inet implementation if reverse DNS does not work.
	 * 
	 * Based on code found at http://www.codingforums.com/showpost.php?p=892349&postcount=5
	 * 
	 * @param ip The IP address to look up
	 * @return   The host name, if one could be found, or the IP address
	 */
	public final static String validatePTR(final String ip) throws UnknownHostException {
	   String retVal = null;
	   if (isValidIP(ip) && !InetAddress.getByName(ip).isLoopbackAddress()) { // we validate only valid IPv4's
		   final String[] bytes = ip.split("\\.");
		   if (bytes.length == 4){
			   try {
			     final java.util.Hashtable<String, String> env = new java.util.Hashtable<String, String>();
			     env.put("java.naming.factory.initial", "com.sun.jndi.dns.DnsContextFactory");
			     final javax.naming.directory.DirContext ctx = new javax.naming.directory.InitialDirContext(env);
			     final String reverseDnsDomain = bytes[3] + "." + bytes[2] + "." + bytes[1] + "." + bytes[0] + ".in-addr.arpa";
			     final javax.naming.directory.Attributes attrs = ctx.getAttributes(reverseDnsDomain, new String[]{"PTR"});
			     final javax.naming.directory.Attributes attrsall = ctx.getAttributes(reverseDnsDomain, null);
			     logger.debug("fortifyHostName check for :: " + ip + " :: Attrs :: " + attrsall);
		         for (final javax.naming.NamingEnumeration<? extends javax.naming.directory.Attribute> ae = attrs.getAll(); ae.hasMoreElements();)
		         {
		            final javax.naming.directory.Attribute attr = ae.next();
		            final String attrId = attr.getID();
		            for (final java.util.Enumeration<?> vals = attr.getAll(); vals.hasMoreElements();)
		            {
		               String value = vals.nextElement().toString();
		               if ("PTR".equals(attrId)) {
		                  final int len = value.length();
		                  if (value.charAt(len - 1) == '.') {
		                     // Strip out trailing period
		                     value = value.substring(0, len - 1);
		                  }
		                  retVal = value;
		               }
		            }
		         }
		         ctx.close();
			   }
			   catch (final javax.naming.NamingException e) {
			         // No reverse DNS that we could find, try with InetAddress
			         System.out.print("\\nNo reverse DNS that we could find :: "+e+" :: were trying with InetAddress :: " + ip); // NO-OP
			   }
			}
	   }
	   else
		   retVal = ip; //dns, ipv6, etc
	   /*
       *	No fallback now, hold on to your seats.
	   *	ArM - #FortifyAuthentication - 07.31.2018
       *

	   {
	      try
	      {
	         retVal = java.net.InetAddress.getByName(ip).getCanonicalHostName(); //ArM - #FortifyOnlyIF
	      }
	      catch (final java.net.UnknownHostException e1)
	      {
	         retVal = ip;
	      }
	   }
	    * 
	    */
	   logger.debug("FortifyIP :: came in with: " + ip + " Got:" + retVal);
	   return retVal;
	 }
	
    public static final String fortifyValidate(String str) {
    	String result = str!=null?str:null;
    	return result;
    }
}
