package gov.va.med.vistalink.adapter.record;

import gov.va.med.exception.FoundationsException;

/**
 * Exception encapsulates Fault information coming from M side.
 * <p>
 * M side can pass bak to Java error condition in the system Fault message. If this condition happens, VLJ creates
 * VistaLinkFaultException and populates it's properties with data from the Fault message.
 * <p>
 * All other Fault exceptions are subclassed from this exception.
 * 
 */

public class VistaLinkFaultException extends FoundationsException {

	// !!! CAUTION !!!
	// If adding new members, make sure to update
	// COPY constructor
	/**
	 * FaultCode from the M side Fault message.
	 */
	private String faultCode;
	/**
	 * FaultString from the M side Fault message.
	 */
	private String faultString;
	/**
	 * FaultActor from the M side Fault message.
	 */
	private String faultActor;
	/**
	 * errorCode from the M side Fault message.
	 */
	private String errorCode;
	/**
	 * errorType from the M side Fault message.
	 */
	private String errorType;
	/**
	 * errorMessage from the M side Fault message.
	 */
	private String errorMessage;

	/**
	 * Constructor for VistaLinkFaultException.
	 * 
	 * @see java.lang.Object#Object()
	 * @va.exclude
	 */
	public VistaLinkFaultException() {
		super();
	}

	/**
	 * Constructor for VistaLinkFaultException.
	 * 
	 * @see java.lang.Throwable#Throwable(java.lang.String)
	 * @va.exclude
	 */
	public VistaLinkFaultException(String msg) {
		super(msg);
	}

	/**
	 * Passed-in parameter values are stored as members of the exception. Exception message string is constructed from
	 * the parameters.
	 * 
	 * @param errorCode
	 * @param errorMessage
	 * @param errorType
	 * @param faultActor
	 * @param faultCode
	 * @param faultString
	 * @va.exclude
	 */
	public VistaLinkFaultException(String errorCode, String errorMessage, String errorType, String faultActor,
			String faultCode, String faultString) {

		super((new StringBuffer(2000)).append("\n\t\tFault Code: '").append(faultCode).append("'; ").append(
				"Fault String: '").append(faultString).append("'; ").append("Fault Actor: '").append(faultActor)
				.append("'; ").append("\n\t\tCode: '").append(errorCode).append("'; ").append("Type: '").append(
						errorType).append("'; ").append("Message: '").append(errorMessage).append("'").toString());
		this.errorCode = errorCode;
		this.errorMessage = errorMessage;
		this.errorType = errorType;
		this.faultActor = faultActor;
		this.faultCode = faultCode;
		this.faultString = faultString;
	}

	/**
	 * Constructor for VistaLinkFaultException.
	 * 
	 * @see gov.va.med.exception.FoundationsException#FoundationsException(java.lang.Throwable)
	 * @va.exclude
	 */
	public VistaLinkFaultException(Exception nestedException) {
		super(nestedException);
	}

	/**
	 * Constructor for VistaLinkFaultException.
	 * 
	 * @see gov.va.med.exception.FoundationsException#FoundationsException(java.lang.String, java.lang.Throwable)
	 * @va.exclude
	 */
	public VistaLinkFaultException(String msg, Exception nestedException) {
		super(msg, nestedException);
	}

	/**
	 * Copy constructor for VistaLinkFaultException.
	 * <p>
	 * This is THE ONLY constructor that subclasses need to implement.
	 * 
	 * @see gov.va.med.exception.FoundationsException#FoundationsException(java.lang.String, java.lang.Throwable)
	 * @va.exclude
	 */
	protected VistaLinkFaultException(VistaLinkFaultException vistaLinkFaultException) {
		super(vistaLinkFaultException.getOriginalMessage(), vistaLinkFaultException.getNestedException());
		this.faultCode = vistaLinkFaultException.getFaultCode();
		this.faultString = vistaLinkFaultException.getFaultString();
		this.faultActor = vistaLinkFaultException.getFaultActor();
		this.errorCode = vistaLinkFaultException.getErrorCode();
		this.errorType = vistaLinkFaultException.getErrorType();
		this.errorMessage = vistaLinkFaultException.getErrorMessage();
	}

	/**
	 * Returns the errorCode.
	 * 
	 * @return String
	 */
	final public String getErrorCode() { //ArM - #FortifyCodeCorectnessOverridableMethod
		return errorCode;
	}

	/**
	 * Returns the errorMessage.
	 * 
	 * @return String
	 */
	final public String getErrorMessage() { //ArM - #FortifyCodeCorectnessOverridableMethod
		return errorMessage;
	}

	/**
	 * Returns the errorType.
	 * 
	 * @return String
	 */
	final public String getErrorType() { //ArM - #FortifyCodeCorectnessOverridableMethod
		return errorType;
	}

	/**
	 * Returns the faultActor.
	 * 
	 * @return String
	 */
	final public String getFaultActor() { //ArM - #FortifyCodeCorectnessOverridableMethod
		return faultActor;
	}

	/**
	 * Returns the faultCode.
	 * 
	 * @return String
	 */
	final public String getFaultCode() { //ArM - #FortifyCodeCorectnessOverridableMethod
		return faultCode;
	}

	/**
	 * Returns the faultString.
	 * 
	 * @return String
	 */
	final public String getFaultString() { //ArM - #FortifyCodeCorectnessOverridableMethod
		return faultString;
	}

}