package gov.va.med.vistalink.adapter.spi;

import gov.va.med.exception.ExceptionUtils;

import javax.resource.ResourceException;
import javax.resource.spi.ConnectionEvent;
import javax.resource.spi.ConnectionEventListener;

//ArM - 06/20/2018 - #TRM
import org.apache.logging.log4j.Logger;
import org.apache.logging.log4j.LogManager;

/**
 * Represents the default(J2SE) ConnectionEventListener that listens for the
 * connectionClosed() and connectionErrorOccurred events fired by
 * VistaLinkManagedConnection. This class is not be used in J2EE, as application
 * servers provide their own implementation.
 * 
 */
public class VistaLinkManagedConnectionEventListener implements ConnectionEventListener {

	/**
	 * The logger for this class
	 */
	private static final Logger logger = LogManager.getLogger(VistaLinkManagedConnectionEventListener.class);

	/**
	 * Constructor for VistaLinkManagedConnectionEventListener.
	 */
	public VistaLinkManagedConnectionEventListener() {
		super();
	}

	/**
	 * Destroys the VistaLinkManagedConnection
	 * 
	 * @see javax.resource.spi.ConnectionEventListener#connectionClosed(javax.resource.spi.ConnectionEvent)
	 */
	public void connectionClosed(ConnectionEvent ce) {
		try {
			((VistaLinkManagedConnection) ce.getSource()).destroy();
			if (logger.isDebugEnabled()) {
				logger.debug("VistaLinkManagedConnection destroyed via close() event and VistaLinkManagedConnectionEventListener");
			}
		} catch (ResourceException e) {

			if (logger.isErrorEnabled()) {
				String errMsg = (new StringBuffer()).append(
						"Error occured while handling managedConnection close() event").append("\n\t").append(
						ExceptionUtils.getFullStackTrace(e)).toString();

				logger.error(errMsg);
			}

		}
	}

	/**
	 * @see javax.resource.spi.ConnectionEventListener#localTransactionStarted(javax.resource.spi.ConnectionEvent)
	 */
	public void localTransactionStarted(ConnectionEvent arg0) {
	}

	/**
	 * @see javax.resource.spi.ConnectionEventListener#localTransactionCommitted(javax.resource.spi.ConnectionEvent)
	 */
	public void localTransactionCommitted(ConnectionEvent arg0) {
	}

	/**
	 * @see javax.resource.spi.ConnectionEventListener#localTransactionRolledback(javax.resource.spi.ConnectionEvent)
	 */
	public void localTransactionRolledback(ConnectionEvent arg0) {
	}

	/**
	 * Destroys the VistaLinkManagedConnection
	 * 
	 * @see javax.resource.spi.ConnectionEventListener#connectionErrorOccurred(javax.resource.spi.ConnectionEvent)
	 */
	public void connectionErrorOccurred(ConnectionEvent ce) {
		try {

			VistaLinkManagedConnection mc = (VistaLinkManagedConnection) ce.getSource();
			mc.cleanup();
			mc.destroy();

		} catch (ResourceException e) {
			if (logger.isErrorEnabled()) {

				String errMsg = (new StringBuffer()).append(
						"Error occured while handling managedConnection error() event").append("\n\t").append(
						ExceptionUtils.getFullStackTrace(e)).toString();

				logger.error(errMsg);
			}

		}
	}

}