package gov.va.med.vistalink.security;

import javax.security.auth.callback.Callback;

/**
 * VistaLoginModule callback for external interactions in the Commit phase of a login.
 * @see VistaLoginModule
 */
final class CallbackCommit implements Callback {

	private String nameNewPerson01;
	private boolean wasCancelPressed;
	private String errorMessageToReport;
	private String ccowSecurePasscode;
	private String kernelCcowToken;
	private String domainName;
	private String vpid;

	/**
	 * Instantiates this callback
	 * @param nameNewPerson01 the name of the person logging in, from the New Person .01 field
	 * @param domainName (not currently used)
	 * @param vpid the user vpid
	 * @param ccowSecurePasscode the secure passcode for accessing the CCOW vault
	 * @param kernelCcowToken the ccow token generated by Kernel on the associated M system
	 */
	CallbackCommit(String nameNewPerson01, String domainName, String vpid, String ccowSecurePasscode,
			String kernelCcowToken) {
		this.nameNewPerson01 = nameNewPerson01;
		this.ccowSecurePasscode = ccowSecurePasscode;
		this.kernelCcowToken = kernelCcowToken;
		this.wasCancelPressed = false;
		this.errorMessageToReport = "";
		this.vpid = vpid;
		this.domainName = domainName;
	}

	String getCcowSecurePasscode() {
		return this.ccowSecurePasscode;
	}

	void setCcowSecurePasscode(String ccowSecurePasscode) {
		this.ccowSecurePasscode = ccowSecurePasscode;
	}

	String getKernelCcowToken() {
		return this.kernelCcowToken;
	}

	void setKernelCcowToken(String kernelCcowToken) {
		this.kernelCcowToken = kernelCcowToken;
	}

	/**
	 * set a person name for later retrieval
	 * @return the person name from the New Person .01 field
	 */
	String getNameNewPerson01() {
		return this.nameNewPerson01;
	}

	// Note: 1/23/06 removed authenticatedConnection field and its Getter; there was no way for it to be set in either a
	// Setter or a constructor, so assume it is a unused field.

	/**
	 * set the "was cancel pressed value" to true or false.
	 * @param value set to true if the user pressed cancel; false otherwise. Defaults to false.
	 */
	void setCancelPressed(boolean value) {
		this.wasCancelPressed = value;
	}

	/**
	 * returns true if the "commit" process was terminated by the user pressing cancel, false otherwise.
	 * @return whether the user pressed cancel
	 */
	boolean wasCancelPressed() {
		return this.wasCancelPressed;
	}

	/**
	 * Sets an error message encountered during callback phase. Doesn't mean login should cancel.
	 * @param errorMessage the error message to report
	 */
	void setErrorMessageToReport(String errorMessage) {
		this.errorMessageToReport = errorMessage;
	}

	/**
	 * Gets any error message to report after the callback commits.
	 * @return error message to report.
	 */
	String getErrorMessageToReport() {
		return this.errorMessageToReport;
	}

	/**
	 * (not currently used -- was formerly used by the CCOW callback handler, before switching to the
	 * using the second ^-piece of the return value from XUS GET CCOW TOKEN.
	 * @return domainName
	 * 
	 */
	String getDomainName() {
		return this.domainName;
	}

	/**
	 * 
	 * @return vpid
	 */
	String getVpid() {
		return this.vpid;
	}
}
